@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2009	PgPool Global Development Group
+ * Copyright (c) 2003-2010	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -84,6 +84,8 @@
 static void daemonize(void);
 static int read_pid_file(void);
 static void write_pid_file(void);
+static int read_status_file(void);
+static int write_status_file(void);
 static pid_t pcp_fork_a_child(int unix_fd, int inet_fd, char *pcp_conf_file);
 static pid_t fork_a_child(int unix_fd, int inet_fd, int id);
 static int create_unix_domain_socket(struct sockaddr_un un_addr_tmp);
@@ -163,6 +165,8 @@ static int pipe_fds[2]; /* for delivering signals */
 
 int my_proc_id;
 
+static BackendStatusRecord backend_rec;	/* Backend status record */
+
 int myargc;
 char **myargv;
 
@@ -262,6 +266,12 @@ int main(int argc, char **argv)
 		}
 	}
 
+#ifdef USE_SSL
+	/* global ssl init */
+	SSL_library_init();
+	SSL_load_error_strings();
+#endif /* USE_SSL */
+
 	mypid = getpid();
 
 	if (pool_init_config())
@@ -356,6 +366,11 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	/*
+	 * Restore previous backend status if possible
+	 */
+	read_status_file();
+
 	/* clear cache */
 	if (clear_cache && pool_config->enable_query_cache && SYSDB_STATUS == CON_UP)
 	{
@@ -469,7 +484,7 @@ int main(int argc, char **argv)
 		myexit(1);
 	}
 
-	pool_log("pgpool successfully started");
+	pool_log("%s successfully started. version %s (%s)", PACKAGE, VERSION, PGPOOLVERSION);
 
 	/* fork a child for PCP handling */
 	snprintf(pcp_un_addr.sun_path, sizeof(pcp_un_addr.sun_path), "%s/.s.PGSQL.%d",
@@ -647,7 +662,7 @@ static void usage(void)
 	fprintf(stderr, "Shutdown modes are:\n");
 	fprintf(stderr, "  smart       quit after all clients have disconnected\n");
 	fprintf(stderr, "  fast        quit directly, with proper shutdown\n");
-	fprintf(stderr, "  immediate   quit without complete shutdown; will lead to recovery on restart\n");
+	fprintf(stderr, "  immediate   the same mode as fast\n");
 }
 
 /*
@@ -782,6 +797,92 @@ static void write_pid_file(void)
 	}
 }
 
+/*
+* Read the status file
+*/
+static int read_status_file(void)
+{
+	FILE *fd;
+	char fnamebuf[POOLMAXPATHLEN];
+	int i;
+	bool someone_wakeup = false;
+
+	snprintf(fnamebuf, sizeof(fnamebuf), "%s/%s", pool_config->logdir, STATUS_FILE_NAME);
+	fd = fopen(fnamebuf, "r");
+	if (!fd)
+	{
+		pool_log("Backend status file %s does not exist", fnamebuf);
+		return -1;
+	}
+	if (fread(&backend_rec, 1, sizeof(backend_rec), fd) != sizeof(backend_rec))
+	{
+		pool_error("Could not read backend status file as %s. reason: %s",
+				   fnamebuf, strerror(errno));
+		fclose(fd);
+		return -1;
+	}
+	fclose(fd);
+
+	for (i=0;i< pool_config->backend_desc->num_backends;i++)
+	{
+		if (backend_rec.status[i] == CON_DOWN)
+			BACKEND_INFO(i).backend_status = CON_DOWN;
+		else
+		{
+			BACKEND_INFO(i).backend_status = CON_CONNECT_WAIT;
+			someone_wakeup = true;
+		}
+	}
+
+	/*
+	 * If no one woke up, we regard the status file bogus
+	 */
+	if (someone_wakeup == false)
+	{
+		for (i=0;i< pool_config->backend_desc->num_backends;i++)
+		{
+			BACKEND_INFO(i).backend_status = CON_CONNECT_WAIT;
+		}
+	}
+
+	return 0;
+}
+
+/*
+* Write the pid file
+*/
+static int write_status_file(void)
+{
+	FILE *fd;
+	char fnamebuf[POOLMAXPATHLEN];
+	int i;
+
+	snprintf(fnamebuf, sizeof(fnamebuf), "%s/%s", pool_config->logdir, STATUS_FILE_NAME);
+	fd = fopen(fnamebuf, "w");
+	if (!fd)
+	{
+		pool_error("Could not open status file %s", fnamebuf);
+		return -1;
+	}
+
+	memset(&backend_rec, 0, sizeof(backend_rec));
+
+	for (i=0;i< pool_config->backend_desc->num_backends;i++)
+	{
+		backend_rec.status[i] = BACKEND_INFO(i).backend_status;
+	}
+
+	if (fwrite(&backend_rec, 1, sizeof(backend_rec), fd) != sizeof(backend_rec))
+	{
+		pool_error("Could not write backend status file as %s. reason: %s",
+				   fnamebuf, strerror(errno));
+		fclose(fd);
+		return -1;
+	}
+	fclose(fd);
+	return 0;
+}
+
 /*
  * fork a child for PCP
  */
@@ -1002,6 +1103,8 @@ static void myexit(int code)
 	myunlink(pcp_un_addr.sun_path);
 	myunlink(pool_config->pid_file_name);
 
+	write_status_file();
+
 	pool_shmem_exit(code);
 	exit(code);
 }
@@ -1403,6 +1506,8 @@ int health_check(void)
 {
 	int fd;
 	int sts;
+	static bool is_first = true;
+	static char *dbname;
 
 	/* V2 startup packet */
 	typedef struct {
@@ -1413,13 +1518,21 @@ int health_check(void)
 	char kind;
 	int i;
 
+	/* Do not execute health check during recovery */
 	if (*InRecovery)
 		return 0;
 
+ Retry:
+	/*
+	 * First we try with "postgres" database.
+	 */
+	if (is_first)
+		dbname = "postgres";
+
 	memset(&mysp, 0, sizeof(mysp));
 	mysp.len = htonl(296);
 	mysp.sp.protoVersion = htonl(PROTO_MAJOR_V2 << 16);
-	strcpy(mysp.sp.database, "template1");
+	strcpy(mysp.sp.database, dbname);
 	strncpy(mysp.sp.user, pool_config->health_check_user, sizeof(mysp.sp.user) - 1);
 	*mysp.sp.options = '\0';
 	*mysp.sp.unused = '\0';
@@ -1434,9 +1547,9 @@ int health_check(void)
 			continue;
 
 		if (*(BACKEND_INFO(i).backend_hostname) == '\0')
-			fd = connect_unix_domain_socket(i);
+			fd = connect_unix_domain_socket(i, FALSE);
 		else
-			fd = connect_inet_domain_socket(i);
+			fd = connect_inet_domain_socket(i, FALSE);
 
 		if (fd < 0)
 		{
@@ -1481,12 +1594,24 @@ int health_check(void)
 			return i+1;
 		}
 
+		if (is_first)
+			is_first = false;
+
 		/*
 		 * If a backend raised a FATAL error(max connections error or
 		 * starting up error?), do not send a Terminate message.
 		 */
 		if ((kind != 'E') && (write(fd, "X", 1) < 0))
 		{
+			if (!strcmp(dbname, "postgres"))
+			{
+				/*
+				 * Retry with template1
+				 */
+				dbname = "template1";
+				goto Retry;
+			}
+
 			pool_error("health check failed during write. host %s at port %d is down. reason: %s. Perhaps wrong health check user?",
 					   BACKEND_INFO(i).backend_hostname,
 					   BACKEND_INFO(i).backend_port,
@@ -1534,9 +1659,9 @@ system_db_health_check(void)
 		return 0;
 
 	if (*SYSDB_INFO->hostname == '\0')
-		fd = connect_unix_domain_socket_by_port(SYSDB_INFO->port, pool_config->backend_socket_dir);
+		fd = connect_unix_domain_socket_by_port(SYSDB_INFO->port, pool_config->backend_socket_dir, FALSE);
 	else
-		fd = connect_inet_domain_socket_by_port(SYSDB_INFO->hostname, SYSDB_INFO->port);
+		fd = connect_inet_domain_socket_by_port(SYSDB_INFO->hostname, SYSDB_INFO->port, FALSE);
 
 	if (fd < 0)
 	{
