@@ -1,12 +1,12 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.48 2009/12/06 12:55:08 t-ishii Exp $
+ * $Header$
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2009	PgPool Global Development Group
+ * Copyright (c) 2003-2010	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -35,6 +35,12 @@
 #include <sys/types.h>
 #include <limits.h>
 
+#ifdef USE_SSL
+#include <openssl/crypto.h>
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#endif
+
 /* undef this if you have problems with non blocking accept() */
 #define NONE_BLOCK
 
@@ -199,6 +205,8 @@ typedef struct {
 	char *system_db_user;		/* user name to access system DB */
 	char *system_db_password;	/* password to access system DB */
 
+	char *lobj_lock_table;		/* table name to lock for rewriting lo_creat */
+
 	BackendDesc *backend_desc;	/* PostgreSQL Server description. Placed on shared memory */
 
 	LOAD_BALANCE_STATUS	load_balance_status[MAX_NUM_BACKENDS];	/* to remember which DB node is selected for load balancing */
@@ -208,6 +216,13 @@ typedef struct {
 	int replication_enabled;		/* replication mode enabled */
 	int master_slave_enabled;		/* master/slave mode enabled */
 	int num_reset_queries;		/* number of queries in reset_query_list */
+
+	/* ssl configuration */
+	int ssl;	/* if non 0, activate ssl support (frontend+backend) */
+	char *ssl_cert;	/* path to ssl certificate (frontend only) */
+	char *ssl_key;	/* path to ssl key (frontend only) */
+	char *ssl_ca_cert;	/* path to root (CA) certificate */
+	char *ssl_ca_cert_dir;	/* path to directory containing CA certificates */
 } POOL_CONFIG;
 
 #define MAX_PASSWORD_SIZE		1024
@@ -228,6 +243,12 @@ typedef struct {
 	int wbufsz;	/* write buffer size */
 	int wbufpo;	/* buffer offset */
 
+#ifdef USE_SSL
+	SSL_CTX *ssl_ctx; /* SSL connection context */
+	SSL *ssl;	/* SSL connection */
+#endif
+	int ssl_active; /* SSL is failed if < 0, off if 0, on if > 0 */
+
 	char *hp;	/* pending data buffer head address */
 	int po;		/* pending data offset */
 	int bufsz;	/* pending data buffer size */
@@ -477,6 +498,7 @@ extern POOL_REQUEST_INFO *Req_info;
 extern volatile sig_atomic_t *InRecovery;
 extern char remote_ps_data[];		/* used for set_ps_display */
 extern volatile sig_atomic_t got_sighup;
+extern volatile sig_atomic_t exit_request;
 
 #define QUERY_STRING_BUFFER_LEN 1024
 extern char query_string_buffer[];		/* last query string sent to simpleQuery() */
@@ -508,8 +530,7 @@ extern int select_load_balancing_node(void);
 extern int pool_init_cp(void);
 extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 									  POOL_CONNECTION_POOL *backend,
-									  int connection_reuse,
-									  int first_ready_for_query_received);
+									  int reset_request);
 
 extern POOL_CONNECTION *pool_open(int fd);
 extern void pool_close(POOL_CONNECTION *cp);
@@ -531,6 +552,14 @@ extern POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMa
 extern void pool_discard_cp(char *user, char *database, int protoMajor);
 extern void pool_backend_timer(void);
 
+/* SSL functionality */
+extern void pool_ssl_negotiate_serverclient(POOL_CONNECTION *cp);
+extern void pool_ssl_negotiate_clientserver(POOL_CONNECTION *cp);
+extern void pool_ssl_close(POOL_CONNECTION *cp);
+extern int pool_ssl_read(POOL_CONNECTION *cp, void *buf, int size);
+extern int pool_ssl_write(POOL_CONNECTION *cp, const void *buf, int size);
+extern bool pool_ssl_pending(POOL_CONNECTION *cp);
+
 extern POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend, 
 								  POOL_CONNECTION_POOL *backend);
 
@@ -544,10 +573,10 @@ extern void send_failback_request(int node_id);
 extern void pool_connection_pool_timer(POOL_CONNECTION_POOL *backend);
 extern RETSIGTYPE pool_backend_timer_handler(int sig);
 
-extern int connect_inet_domain_socket(int secondary_backend);
-extern int connect_unix_domain_socket(int secondary_backend);
-extern int connect_inet_domain_socket_by_port(char *host, int port);
-extern int connect_unix_domain_socket_by_port(int port, char *socket_dir);
+extern int connect_inet_domain_socket(int slot, bool retry);
+extern int connect_unix_domain_socket(int slot, bool retry);
+extern int connect_inet_domain_socket_by_port(char *host, int port, bool retry);
+extern int connect_unix_domain_socket_by_port(int port, char *socket_dir, bool retry);
 
 extern void pool_set_timeout(int timeoutval);
 extern int pool_check_fd(POOL_CONNECTION *cp);
@@ -619,8 +648,6 @@ extern POOL_STATUS OneNode_do_command(POOL_CONNECTION *frontend, POOL_CONNECTION
 extern POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	char *hostname, int port, char *dbname, char *user, char *password);
 
-extern POOL_STATUS do_query(POOL_CONNECTION *backend, char *query, POOL_SELECT_RESULT **result);
-
 /* define pool_system.c */
 extern POOL_CONNECTION_POOL_SLOT *pool_system_db_connection(void);
 extern DistDefInfo *pool_get_dist_def_info (char * dbname, char * schema_name, char * table_name);
@@ -665,17 +692,26 @@ extern void cancel_request(CancelPacket *sp);
 extern void check_stop_request(void);
 
 /* pool_process_query.c */
-void free_select_result(POOL_SELECT_RESULT *result);
-void reset_variables(void);
-void per_node_statement_log(POOL_CONNECTION_POOL *backend, int node_id, char *query);
+extern void reset_variables(void);
+extern void reset_connection(void);
+extern void per_node_statement_log(POOL_CONNECTION_POOL *backend, int node_id, char *query);
+extern void per_node_error_log(POOL_CONNECTION_POOL *backend, int node_id, char *query, char *prefix, bool unread);
+extern int pool_extract_error_message(bool read_kind, POOL_CONNECTION *backend, int major, bool unread, char **message);
+extern POOL_STATUS do_command(POOL_CONNECTION *frontend, POOL_CONNECTION *backend,
+					   char *query, int protoMajor, int pid, int key, int no_ready_for_query);
+extern POOL_STATUS do_query(POOL_CONNECTION *backend, char *query, POOL_SELECT_RESULT **result, int major);
+extern void free_select_result(POOL_SELECT_RESULT *result);
 
 /* pool_relcache.c */
-POOL_RELCACHE *pool_create_relcache(int cachesize, char *sql,
+extern POOL_RELCACHE *pool_create_relcache(int cachesize, char *sql,
 									func_ptr register_func, func_ptr unregister_func,
 									bool issessionlocal);
-void pool_discard_relcache(POOL_RELCACHE *relcache);
-void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backend, char *table);
-void *int_register_func(POOL_SELECT_RESULT *res);
-void *int_unregister_func(void *data);
+extern void pool_discard_relcache(POOL_RELCACHE *relcache);
+extern void *pool_search_relcache(POOL_RELCACHE *relcache, POOL_CONNECTION_POOL *backend, char *table);
+extern void *int_register_func(POOL_SELECT_RESULT *res);
+extern void *int_unregister_func(void *data);
+
+/* pool_lobj.c */
+extern char *pool_rewrite_lo_creat(char kind, char *packet, int packet_len, POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, int* len);
 
 #endif /* POOL_H */
