@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_process_reporting.c,v 1.2 2009/12/06 08:46:34 t-ishii Exp $
+ * $Header$
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2009	PgPool Global Development Group
+ * Copyright (c) 2003-2010	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -52,9 +52,15 @@ void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 		char desc[POOLCONFIG_MAXDESCLEN+1];
 	} POOL_REPORT_STATUS;
 
-#define MAXITEMS 128
+/*
+ * Report data buffer.
+ * 128 is the max number of configuration items.
+ * In addition, we need MAX_NUM_BACKENDS*4
+ * for backend descriptions.
+ */
+#define MAXITEMS (128 + MAX_NUM_BACKENDS*4)		
 
-	POOL_REPORT_STATUS status[MAXITEMS];
+	static POOL_REPORT_STATUS status[MAXITEMS];
 
 	short nrows;
 	int size;
@@ -300,6 +306,26 @@ void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 	strncpy(status[i].desc, "if idle for this seconds, child connection closes in recovery 2nd statge", POOLCONFIG_MAXDESCLEN);
 	i++;
 
+	strncpy(status[i].name, "lobj_lock_table", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->lobj_lock_table);
+	strncpy(status[i].desc, "table name used for large object replication control", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	strncpy(status[i].name, "ssl", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->ssl);
+	strncpy(status[i].desc, "SSL support", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	strncpy(status[i].name, "ssl_key", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->ssl_key);
+	strncpy(status[i].desc, "path to the SSL private key file", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	strncpy(status[i].name, "ssl_cert", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->ssl_cert);
+	strncpy(status[i].desc, "path to the SSL public certificate file", POOLCONFIG_MAXDESCLEN);
+	i++;
+
 	strncpy(status[i].name, "parallel_mode", POOLCONFIG_MAXNAMELEN);
 	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->parallel_mode);
 	strncpy(status[i].desc, "if non 0, run in parallel query mode", POOLCONFIG_MAXDESCLEN);
