@@ -45,6 +45,7 @@
 
 #include "pool.h"
 #include "pool_signal.h"
+#include "pool_timestamp.h"
 #include "pool_proto_modules.h"
 #include "parser/pool_string.h"
 
@@ -346,14 +347,14 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 			return POOL_CONTINUE;
 		}
 
-		if (IsA(node, PrepareStmt) ||
+		if (IsA(node, PrepareStmt) || IsA(node, DeallocateStmt) ||
 			IsA(node, VariableSetStmt) || IsA(node, DiscardStmt))
 		{
 			/*
 			 * PREPARE, SET and DISCARD statements must be
 			 * replicated even if we are in master/slave mode.
 			 */
-			if (MASTER_SLAVE && TSTATE(backend) != 'E')
+			if (MASTER_SLAVE && TSTATE(backend) != 'E' && ! IsA(node, DeallocateStmt))
 				force_replication = 1;
 
 			/*
@@ -526,17 +527,106 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 	{
 		/* check if query is "COMMIT" or "ROLLBACK" */
 		commit = is_commit_query(node);
-		free_parser();
 
 		/*
 		 * Query is not commit/rollback
 		 */
 		if (!commit)
 		{
+			char		*rewrite_query;
+
+			if (node)
+		   	{
+				Portal *portal = NULL;
+
+				if (IsA(node, PrepareStmt))
+				{
+					portal = pending_prepared_portal;
+					portal->num_tsparams = 0;
+				}
+				else if (IsA(node, ExecuteStmt))
+					portal = lookup_prepared_statement_by_statement(
+							&prepared_list, ((ExecuteStmt *) node)->name);
+
+				/* rewrite `now()' to timestamp literal */
+				rewrite_query = rewrite_timestamp(backend, node, false, portal);
+				if (rewrite_query != NULL)
+				{
+					string = rewrite_query;
+					len = strlen(string) + 1;
+				}
+
+			}
+
+			/*
+			 * Optimization effort: If there's only one session, we do
+			 * not need to wait for the master node's response, and
+			 * could execute a query concurrently.
+			 */
+			if (pool_config->num_init_children == 1)
+			{
+				/* Send query to DB nodes */
+				for (i=0;i<NUM_BACKENDS;i++)
+				{
+					if (!VALID_BACKEND(i))
+						continue;
+
+					per_node_statement_log(backend, i, string);
+
+					if (send_simplequery_message(CONNECTION(backend, i), len, string, MAJOR(backend)) != POOL_CONTINUE)
+					{
+						free_parser();
+						return POOL_END;
+					}
+				}
+
+				/* Wait for response from DB nodes */
+				for (i=0;i<NUM_BACKENDS;i++)
+				{
+					if (!VALID_BACKEND(i))
+						continue;
+
+					if (wait_for_query_response(frontend, CONNECTION(backend, i), string, MAJOR(backend)) != POOL_CONTINUE)
+					{
+						/* Cancel current transaction */
+						CancelPacket cancel_packet;
+
+						cancel_packet.protoVersion = htonl(PROTO_CANCEL);
+						cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
+						cancel_packet.key= MASTER_CONNECTION(backend)->key;
+						cancel_request(&cancel_packet);
+
+						free_parser();
+						return POOL_END;
+					}
+
+					/*
+					 * Check if some error detected.  If so, emit
+					 * log. This is usefull when invalid encoding error
+					 * occurs. In this case, PostgreSQL does not report
+					 * what statement caused that error and make users
+					 * confused.
+					 */
+					per_node_error_log(backend, i, string, "SimpleQuery: Error or notice message from backend: ", true);
+
+				}
+				if (commit)
+				{
+					TSTATE(backend) = 'I';
+				}
+				free_parser();
+				return POOL_CONTINUE;
+			}
+
 			/* Send the query to master node */
 
+			per_node_statement_log(backend, MASTER_NODE_ID, string);
+
 			if (send_simplequery_message(MASTER(backend), len, string, MAJOR(backend)) != POOL_CONTINUE)
+			{
+				free_parser();
 				return POOL_END;
+			}
 
 			if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
 			{
@@ -548,6 +638,7 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				cancel_packet.key= MASTER_CONNECTION(backend)->key;
 				cancel_request(&cancel_packet);
 
+				free_parser();
 				return POOL_END;
 			}
 
@@ -562,6 +653,17 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				string = POOL_ERROR_QUERY;
 				len = strlen(string) + 1;
 			}
+			else
+			{
+				/*
+				 * Check if some error detected.  If so, emit
+				 * log. This is usefull when invalid encoding error
+				 * occurs. In this case, PostgreSQL does not report
+				 * what statement caused that error and make users
+				 * confused.
+				 */
+				per_node_error_log(backend, MASTER_NODE_ID, string, "SimpleQuery: Error or notice message from backend: ", true);
+			}
 		}
 
 		/* send query to other than master nodes */
@@ -570,8 +672,13 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 			if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i))
 				continue;
 
+			per_node_statement_log(backend, i, string);
+
 			if (send_simplequery_message(CONNECTION(backend, i), len, string, MAJOR(backend)) != POOL_CONTINUE)
+			{
+				free_parser();
 				return POOL_END;
+			}
 		}
 
 		/* Wait for nodes othan than the master node */
@@ -590,15 +697,31 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				cancel_packet.key= MASTER_CONNECTION(backend)->key;
 				cancel_request(&cancel_packet);
 
+				free_parser();
 				return POOL_END;
 			}
+
+			/*
+			 * Check if some error detected.  If so, emit
+			 * log. This is usefull when invalid encoding error
+			 * occurs. In this case, PostgreSQL does not report
+			 1* what statement caused that error and make users
+			 * confused.
+			 */
+			per_node_error_log(backend, i, string, "SimpleQuery: Error or notice message from backend: ", true);
+
 		}
 
 		/* send "COMMIT" or "ROLLBACK" to only master node if query is "COMMIT" or "ROLLBACK" */
 		if (commit)
 		{
+			per_node_statement_log(backend, MASTER_NODE_ID, string);
+
 			if (send_simplequery_message(MASTER(backend), len, string, MAJOR(backend)) != POOL_CONTINUE)
+			{
+				free_parser();
 				return POOL_END;
+			}
 
 			if (wait_for_query_response(frontend, MASTER(backend), string, MAJOR(backend)) != POOL_CONTINUE)
 			{
@@ -610,16 +733,27 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				cancel_packet.key= MASTER_CONNECTION(backend)->key;
 				cancel_request(&cancel_packet);
 
+				free_parser();
 				return POOL_END;
 			}
 
+			/*
+			 * Check if some error detected.  If so, emit
+			 * log. This is usefull when invalid encoding error
+			 * occurs. In this case, PostgreSQL does not report
+			 1* what statement caused that error and make users
+			 * confused.
+			 */
+			per_node_error_log(backend, MASTER_NODE_ID, string, "SimpleQuery: Error or notice message from backend: ", true);
 
 			TSTATE(backend) = 'I';
 		}
+		free_parser();
 	}
 	else
 	{
-		free_parser();
+		per_node_statement_log(backend, MASTER_NODE_ID, string);
+
 		if (send_simplequery_message(MASTER(backend), len, string, MAJOR(backend)) != POOL_CONTINUE)
 			return POOL_END;
 
@@ -633,8 +767,20 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				cancel_packet.key= MASTER_CONNECTION(backend)->key;
 				cancel_request(&cancel_packet);
 
+				free_parser();
 				return POOL_END;
 		}
+
+		/*
+		 * Check if some error detected.  If so, emit
+		 * log. This is usefull when invalid encoding error
+		 * occurs. In this case, PostgreSQL does not report
+		 1* what statement caused that error and make users
+		 * confused.
+		 */
+		per_node_error_log(backend, MASTER_NODE_ID, string, "SimpleQuery: Error or notice message from backend: ", true);
+
+		free_parser();
 	}
 
 	return POOL_CONTINUE;
@@ -652,7 +798,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 	char kind;
 	int status, commit = 0;
 	Portal *portal;
-	char *string1;
+	char *string1 = NULL;
 	PrepareStmt *p_stmt;
 	POOL_STATUS ret;
 	int specific_error = 0;
@@ -757,7 +903,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 		if (!commit)
 		{
 			/* Send the query to master node */
-
+			per_node_statement_log(backend, MASTER_NODE_ID, string1);
 			if (send_execute_message(backend, MASTER_NODE_ID, len, string) != POOL_CONTINUE)
 				return POOL_END;
 
@@ -799,8 +945,12 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 				if (send_execute_message(backend, i, len + 5, msg))
 					return POOL_END;
 			}
-			else if (send_execute_message(backend, i, len, string) != POOL_CONTINUE)
-				return POOL_END;
+			else
+			{
+				per_node_statement_log(backend, i, string1);
+				if (send_execute_message(backend, i, len, string) != POOL_CONTINUE)
+					return POOL_END;
+			}
 		}
 
 		/* Wait for nodes other than the master node */
@@ -826,6 +976,8 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 		/* send "COMMIT" or "ROLLBACK" to only master node if query is "COMMIT" or "ROLLBACK" */
 		if (commit)
 		{
+			per_node_statement_log(backend, MASTER_NODE_ID, string1);
+
 			if (send_execute_message(backend, MASTER_NODE_ID, len, string) != POOL_CONTINUE)
 				return POOL_END;
 
@@ -845,6 +997,8 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend,
 	}
 	else
 	{
+		per_node_statement_log(backend, MASTER_NODE_ID, string1);
+
 		if (send_execute_message(backend, MASTER_NODE_ID, len, string) != POOL_CONTINUE)
 			return POOL_END;
 
@@ -904,6 +1058,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 	int deadlock_detected = 0;
 	int insert_stmt_with_lock = 0;
 	POOL_STATUS status;
+	char per_node_statement_log_buffer[1024];
 
 	/* read Parse packet */
 	if (pool_read(frontend, &len, sizeof(len)) < 0)
@@ -920,7 +1075,8 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 	parse_tree_list = raw_parser(stmt);
 	if (parse_tree_list == NIL)
 	{
-		free_parser();
+		/* free_parser(); */
+		;
 	}
 	else
 	{
@@ -954,6 +1110,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 		if (portal == NULL)
 		{
 			pool_error("Parse: create_portal() failed");
+			free_parser();
 			return POOL_END;
 		}
 
@@ -988,9 +1145,45 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 			pending_prepared_portal = portal;
 		}
 
-		/* switch old memory context */
+		/*
+		 * Switch to old memory context. Caution. Now we are in parser
+		 * memory context.
+		 * Palloced memories will be gone if free_parser() called!
+		 */
 		pool_memory = old_context;
 
+		if (REPLICATION)
+		{
+			char		*rewrite_query;
+			bool		 rewrite_to_params = true;
+
+			/*
+			 * rewrite `now()'.
+			 * if stmt is unnamed, we rewrite `now()' to timestamp constant.
+			 * else we rewrite `now()' to params and expand that at Bind
+			 * message.
+			 */
+			if (*name == '\0')
+				rewrite_to_params = false;
+			portal->num_tsparams = 0;
+			rewrite_query = rewrite_timestamp(backend, node, rewrite_to_params, portal);
+			if (rewrite_query != NULL)
+			{
+				int alloc_len = len - strlen(stmt) + strlen(rewrite_query);
+				string = palloc(alloc_len);
+				strcpy(string, name);
+				strcpy(string + strlen(name) + 1, rewrite_query);
+				memcpy(string + strlen(name) + strlen(rewrite_query) + 2,
+						stmt + strlen(stmt) + 1,
+						len - (strlen(name) + strlen(stmt) + 2));
+
+				len = len - strlen(stmt) + strlen(rewrite_query);
+				name = string;
+				stmt = string + strlen(name) + 1;
+				pool_debug("rewrite query  %s %s len=%d", name, stmt, len);
+			}
+		}
+
 		if (REPLICATION)
 		{
 			char kind;
@@ -1009,9 +1202,16 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 
 				kind = pool_read_kind(backend);
 				if (kind != 'Z')
+				{
+					free_parser();
 					return POOL_END;
+				}
+
 				if (ReadyForQuery(frontend, backend, 0) != POOL_CONTINUE)
+				{
+					free_parser();
 					return POOL_END;
+				}
 			}
 
 			if (is_strict_query(node))
@@ -1023,17 +1223,27 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 				status = insert_lock(frontend, backend, stmt, (InsertStmt *)node);
 				if (status != POOL_CONTINUE)
 				{
+					free_parser();
 					return status;
 				}
 			}
 		}
-		free_parser();
 	}
 
 	/* send to master node */
+	snprintf(per_node_statement_log_buffer, sizeof(per_node_statement_log_buffer), "Parse: %s", stmt);
+	per_node_statement_log(backend, MASTER_NODE_ID, per_node_statement_log_buffer);
 	if (send_extended_protocol_message(backend, MASTER_NODE_ID,
 									   "P", len, string))
+	{
+		free_parser();
 		return POOL_END;
+	}
+
+	/*
+	 * Cannot call free_parser() here. Since "string" might be allocated in parser context.
+	 * free_parser();
+	 */
 
 	if (REPLICATION || PARALLEL_MODE || MASTER_SLAVE)
 	{
@@ -1051,6 +1261,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 			cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
 			cancel_packet.key= MASTER_CONNECTION(backend)->key;
 			cancel_request(&cancel_packet);
+			free_parser();
 			return POOL_END;
 		}
 
@@ -1062,7 +1273,20 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 		 */
 		deadlock_detected = detect_deadlock_error(MASTER(backend), MAJOR(backend));
 		if (deadlock_detected < 0)
+		{
+			free_parser();
 			return POOL_END;
+		}
+		else
+		{
+			/*
+			 * Check if other than deadlock error detected.  If so, emit
+			 * log. This is usefull when invalid encoding error occurs. In
+			 * this case, PostgreSQL does not report what statement caused
+			 * that error and make users confused.
+			 */
+			per_node_error_log(backend, MASTER_NODE_ID, stmt, "Parse(): Error or notice message from backend: ", true);
+		}
 
 		for (i=0;i<NUM_BACKENDS;i++)
 		{
@@ -1072,15 +1296,28 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 				{
 					pool_log("Parse: received deadlock error message from master node");
 
+					per_node_statement_log(backend, i, POOL_ERROR_QUERY);
+
 					if (send_simplequery_message(CONNECTION(backend, i),
 												 strlen(POOL_ERROR_QUERY)+1,
 												 POOL_ERROR_QUERY,
 												 MAJOR(backend)))
+					{
+						free_parser();
 						return POOL_END;
+					}
+				}
+				else
+				{
+					snprintf(per_node_statement_log_buffer, sizeof(per_node_statement_log_buffer), "Parse: %s", stmt);
+					per_node_statement_log(backend, i, per_node_statement_log_buffer);
+
+					if (send_extended_protocol_message(backend, i,"P", len, string))
+					{
+						free_parser();
+						return POOL_END;
+					}
 				}
-				else if (send_extended_protocol_message(backend, i,
-														"P", len, string))
-					return POOL_END;
 			}
 		}
 
@@ -1100,11 +1337,26 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 				cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
 				cancel_packet.key= MASTER_CONNECTION(backend)->key;
 				cancel_request(&cancel_packet);
+				free_parser();
 				return POOL_END;
 			}
+
+			/*
+			 * Check if error (or notice response) from backend is
+			 * detected.  If so, emit log. This is usefull when
+			 * invalid encoding error occurs. In this case, PostgreSQL
+			 * does not report what statement caused that error and
+			 * make users confused.
+			 */
+			per_node_error_log(backend, i, stmt, "Parse(): Error or notice message from backend: ", true);
 		}
 	}
 
+	/*
+	 * Ok. we are safe to call free_parser();
+	 */
+	free_parser();
+
 	for (;;)
 	{
 		POOL_STATUS ret;
@@ -1114,8 +1366,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 			return ret;
 
 		SimpleForwardToFrontend(kind, frontend, backend);
-		if (pool_flush(frontend) < 0)
-			return POOL_ERROR;
+		pool_flush(frontend);
 
 		/* Ignore warning messages */
 		if (kind != 'N')
@@ -2554,3 +2805,30 @@ static int is_temp_table(POOL_CONNECTION_POOL *backend, Node *node)
 	result = pool_search_relcache(relcache, backend, str)==0?0:1;
 	return result;
 }
+
+/*
+ * Make per DB node statement log
+ */
+void per_node_statement_log(POOL_CONNECTION_POOL *backend, int node_id, char *query)
+{
+	POOL_CONNECTION_POOL_SLOT *slot = backend->slots[node_id];
+
+	if (pool_config->log_per_node_statement)
+		pool_log("DB node id: %d backend pid: %d statement: %s", node_id, ntohl(slot->pid), query);
+}
+
+/*
+ * Check kind and produce error message
+ * All data read in this function is returned to stream.
+ */
+void per_node_error_log(POOL_CONNECTION_POOL *backend, int node_id, char *query, char *prefix, bool unread)
+{
+	POOL_CONNECTION_POOL_SLOT *slot = backend->slots[node_id];
+	char *message;
+
+	if (pool_extract_error_message(true, CONNECTION(backend, node_id), MAJOR(backend), true, &message) > 0)
+	{
+		pool_log("%s: DB node id: %d backend pid: %d statement: %s message: %s",
+				 prefix, node_id, ntohl(slot->pid), query, message);
+	}
+}
